--╔════════════════════════════╗════════════════════════════════════════════════════════════════════════════════#
--║     	  EXAMPLES		   ║  Just some examples you definitely want to remove in your own implementation.
--╚════════════════════════════╝════════════════════════════════════════════════════════════════════════════════#
local function Monitor_MobjThinker(mo)
	if mo and mo.health and not S_SoundPlaying(mo, sfx_laser)
		S_StartSound(mo, sfx_laser)
	end
end

local function Monitor_MobjDeath(mo, inflictor, source)
	if source and source.player
		CONS_Printf(source.player, "You've\x85 murdered\x80 an innocent monitor, congrats.")	
	end
end

local function Monitor_Removed(mo)
	S_StartSound(nil, sfx_mario8)
end

local function Enemy_MobjSpawn(mo)
	mo.renderflags = $|RF_FULLDARK|RF_HORIZONTALFLIP --inb4 OMGRACISM?!1?!!1!
	mo.scale = max($, 2<<16)
	mo.mirrored = true
end

local function Player_MobjSpawn(mo)
	mo.renderflags = $|RF_FULLBRIGHT|RF_NOCOLORMAPS|RF_VERTICALFLIP|RF_HORIZONTALFLIP
	mo.colorized = true
	if mo.player
		mo.player.powers[pw_shield] = SH_FIREFLOWER
		mo.player.skincolor = SKINCOLOR_SUPERSILVER1
		mo.color = SKINCOLOR_SUPERSILVER1
		mo.player.normalspeed = max($, 45<<16)
		mo.player.jumpfactor = max($, 71000) 
		CONS_Printf(mo.player, "A glowing blight spawns upon this world. And they're upsidedown.")	
	end
end

/*
╔════════════════════════════╗═════════════════════════════════════════════════════════════════════════════════#
║     	ACTUAL CODE		     ║  Below is the actual core you want to use. Keep your local functions above it!
╚════════════════════════════╝═════════════════════════════════════════════════════════════════════════════════#
Legend:
type = The object type number. So MT_PLAYER, MT_THOK, MT_RING, etc.
INFO = The mobjinfo field. Same as if you were accessing mo.info.
SemiGlobal = A local table keeping track of which objects got hooks assigned already. Prevent duplication!
*/
local SemiGlobal = {}
local function GSAddonLoad(_)
	for type = 1,max(1, (#mobjinfo)-1) local INFO=mobjinfo[type] if INFO==nil or (SemiGlobal[type]) continue end --
	
		--From here on out, you can specify what objects should get the hook. 
		--Below is just more example code to be replaced with your own.
		
		if (INFO.flags & MF_MONITOR) --For all objects with the MF_MONITOR flag, add a hook!
			SemiGlobal[type] = true --Adding this is crucial! It prevents hooks from being added multiple times.
			addHook("MobjDeath", Monitor_MobjDeath, type) --This is how you should add hooks.
			addHook("MobjRemoved", Monitor_Removed, type)
			addHook("MobjThinker", Monitor_MobjThinker, type)
		end
		
		--For all enemies, or some exceptions that don't have the flag, do this!
		if (INFO.flags & MF_ENEMY) or type==MT_MINUS or type==MT_EGGGUARD
			SemiGlobal[type] = true --Adding this is crucial!
			addHook("MobjSpawn", Enemy_MobjSpawn, type)
		elseif type==MT_PLAYER
			SemiGlobal[type] = true --Adding this is crucial!
			addHook("MobjSpawn", Player_MobjSpawn, type)			
		end
	end
end

addHook("AddonLoaded", GSAddonLoad) --This hook makes sure to scan for new objects everytime a new mod is added.
GSAddonLoad(_) --Run the function once on initial load.

addHook("NetVars", function(network)
	SemiGlobal = network($) --Sync what's been added so far for netgame sanity.
end)